import matplotlib.pyplot as plt

def plot_time_series(series, title='Time Series'):
    """
    Построение графика временного ряда

    series: временной ряд
    title:  заголовок графика
    """

    plt.figure(figsize=(10, 6))
    plt.plot(series, label='Original Series')
    plt.title(title)
    plt.xlabel('Date')
    plt.ylabel('Value')
    plt.legend()
    plt.show()

def plot_smoothed_series(original_series, smoothed_series, title='Smoothed Series'):
    """
    Построение графика сглаженного временного ряда

    original_series: исходный временной ряд
    smoothed_series: сглаженный временной ряд
    title:  заголовок графика
    """

    plt.figure(figsize=(10, 6))
    plt.plot(original_series, label='Original Series')
    plt.plot(smoothed_series, label='Smoothed Series', color='red')
    plt.title(title)
    plt.xlabel('Date')
    plt.ylabel('Value')
    plt.legend()
    plt.show()

def plot_anomalies(series, anomalies, title='Anomalies in Time Series'):
    """
    Построение графика временного ряда с выделением аномалий

    series: временной ряд
    anomalies: временной ряд аномалий
    title:  заголовок графика
    """

    plt.figure(figsize=(10, 6))
    plt.plot(series, color='blue', label='Time Series')
    plt.scatter(anomalies.index, anomalies, color='red', label='Anomalies')
    plt.legend(loc='best')
    plt.title(title)
    plt.show()


import keras


def load_model(filename, model_type):
    """
    Функция для загрузки модели из файла

    filename: имя файла модели
    model_type: тип модели
    """

    if model_type.lower() == 'lstm':
        model = keras.models.load_model(filename)
    else:
        model = None

    return model


def model_forecast(model_fit, model_type, series=None, steps=5, features=1):
    """
    Прогнозирование временного ряда с использованием модели

    model_fit: обученная модель
    model_type: тип модели
    series: времненной ряд для прогнозирования
    steps: количество шагов для прогноза (у timemixer по умолчанию стоит свой размер горизонта прогнозирования)
    features: количество признаков на временной шаг

    """

    if model_type.lower() == 'lstm':
        input_seq = series.iloc[-steps:].values
        input_seq = input_seq.reshape((1, steps, features))
        forecast = model_fit.predict(input_seq, verbose=0)

    elif model_type.lower() == 'timemixer':
        forecast = model_fit.predict()[:steps]
    else:
        forecast = model_fit.forecast(steps=steps)

    return forecast



import numpy as np
import pandas as pd


def prepare_data_tm(df, date_col, target_col):
    """
    Преобразовывает данные для TimeMixer

    df: DataFrame с данными
    date_col: название столбца с датами
    target_col: название столбца с целевой переменной

    return: измененные данные
    """
    df = df.copy()
    df = df.reset_index()
    df.rename(columns={date_col: 'ds', target_col: 'y'}, inplace=True)
    df['unique_id'] = 'series'
    df = df.sort_values('ds')
    columns_to_keep = ['unique_id', 'ds', 'y']

    exog_cols = [col for col in df.columns if col not in ['unique_id', 'ds', 'y']]
    columns_to_keep.extend(exog_cols)

    df = df[columns_to_keep]

    return df


from scipy.signal import savgol_filter

def moving_average(series, window):
    """
    Сглаживание методом скользящего среднего

    series: временной ряд
    window: размер окна скользящего среднего
    """

    return series.rolling(window=window).mean()

def exponential_smoothing(series, alpha):
    """
    Экспоненциальное сглаживание

    series: временной ряд
    alpha: коэффициент сглаживания (от 0 до 1)
    """

    return series.ewm(alpha=alpha).mean()

def savitzky_golay_filter(series, window_length, polyorder):
    """
    Фильтр Савицкого-Голея

    series: временной ряд
    window_length: длина окна
    polyorder: порядок полинома
    """

    return savgol_filter(series, window_length, polyorder)



import numpy as np
import matplotlib.pyplot as plt
from scipy.signal import periodogram


def plot_periodogram(series, fs=1.0):
    """
    Построение периодограммы для спектрального анализа временного ряда

    series: временной ряд
    fs: частота дискретизации
    """

    freqs, spectrum = periodogram(series, fs=fs)

    plt.figure(figsize=(10, 6))
    plt.plot(freqs, spectrum)
    plt.xlabel('Frequency')
    plt.ylabel('Power Spectrum')
    plt.title('Periodogram')
    plt.show()


def plot_fft(series):
    """
    Построение спектра Фурье для временного ряда

    series: временной ряд
    """
    n = len(series)
    fft_result = np.fft.fft(series)
    fft_freqs = np.fft.fftfreq(n)

    plt.figure(figsize=(10, 6))
    plt.plot(fft_freqs[:n // 2], np.abs(fft_result)[:n // 2])
    plt.xlabel('Frequency')
    plt.ylabel('Amplitude')
    plt.title('Fourier Transform')
    plt.show()



import pandas as pd
import matplotlib.pyplot as plt
from app.timeseries_lib import (
    moving_average, exponential_smoothing, savitzky_golay_filter,
    plot_time_series, plot_smoothed_series, plot_anomalies,
    decompose_series, load_model, model_forecast,
    detect_anomalies_iqr, detect_anomalies_z_score,
    ar_model, ma_model, arima_model, sarima_model, holt_winters_model, lstm_model,
    check_stationarity_adf, plot_trend, plot_autocorrelation,
    plot_periodogram, plot_fft
)
import keras

# Создаем пример временного ряда
data = pd.read_csv('Airplane.csv', parse_dates=['Date'])
time_series = data.groupby('Date')['Fatalities'].sum()

# # Сглаживание
# smoothed_ma = moving_average(time_series, window=12)
# smoothed_es = exponential_smoothing(time_series, alpha=0.3)
# smoothed_sg = savitzky_golay_filter(time_series, window_length=11, polyorder=2)
#
# # Визуализация
# plot_time_series(time_series, title='Airplane Crashes Fatalities')
# plot_smoothed_series(time_series, smoothed_ma, title='Moving Average Smoothing')
# plot_smoothed_series(time_series, smoothed_es, title='Exponential Smoothing')
# plot_smoothed_series(time_series, smoothed_sg, title='Savitzky-Golay Smoothing')
#
# # Проверка стационарности
# stationarity_result = check_stationarity_adf(time_series)
# print('ADF Test Result:')
# print(stationarity_result)
#
# # Построение тренда
# period = 12
# plot_trend(time_series, period)
#
# # Построение автокорреляции
# plot_autocorrelation(time_series)
#
# # Декомпозиция
# decomposition = decompose_series(time_series)
# decomposition.plot()
# plt.show()
#
# # Спектральный анализ
# plot_periodogram(time_series)
# plot_fft(time_series)
#
# # Обнаружение аномалий
# anomalies_iqr = detect_anomalies_iqr(time_series)
# print('Anomalies detected using IQR method:')
# print(anomalies_iqr)
#
# anomalies_z_score = detect_anomalies_z_score(time_series)
# print('Anomalies detected using Z-score method:')
# print(anomalies_z_score)
#
# # Визуализация аномалий
# plot_anomalies(time_series, anomalies_iqr, title='Anomalies (IQR method)')
# plot_anomalies(time_series, anomalies_z_score, title='Anomalies (Z-score method)')
#
# # Моделирование
# ar_fit = ar_model(time_series, lags=5)
# print(ar_fit.summary())
#
# ma_fit = ma_model(time_series, lags=5)
# print(ma_fit.summary())
#
# arima_fit = arima_model(time_series, order=(1, 1, 1))
# print(arima_fit.summary())
#
# sarima_fit = sarima_model(time_series, order=(1, 1, 1), seasonal_order=(1, 1, 1, 12))
# print(sarima_fit.summary())
#
# holt_winters_fit = holt_winters_model(time_series, seasonal_periods=12)
# print(holt_winters_fit.summary())

layer_config = [
    {'type': 'LSTM', 'units': 50, 'return_sequences': True, 'bidirectional': True},
    {'type': 'LSTM', 'units': 50, 'return_sequences': False},
    {'type': 'Dense', 'units': 1}
]
lstm_fit = lstm_model(time_series, 10, layer_config, n_features=1, epochs=10, batch_size=1)
lstm_fit_file = load_model('lstm_model_2024-09-15_21-57-45.h5', 'LSTM')
print(lstm_fit_file.summary())

# # Прогнозирование
# forecast_ar = model_forecast(ar_fit, steps=10)
# print('\nAR forecast')
# print(forecast_ar)
#
# forecast_ma = model_forecast(ma_fit, steps=10)
# print('\nMA forecast')
# print(forecast_ma)
#
# forecast_arima = model_forecast(arima_fit, steps=10)
# print('\nARIMA forecast')
# print(forecast_arima)
#
# forecast_sarima = model_forecast(sarima_fit, steps=10)
# print('\nSARIMA forecast')
# print(forecast_sarima)
#
# forecast_holt_winters = model_forecast(arima_fit, steps=10)
# print('\nHolt-Winters forecast')
# print(forecast_holt_winters)

forecast_lstm = model_forecast(lstm_fit_file, time_series, 'LSTM', steps=10)
print('\nLSTM forecast')
print(forecast_lstm)
